// $Id: main.cpp,v 1.1.1.1 2005/04/22 17:18:53 ndsdev Exp $
//////////////////////////////////////////////////////////////////////
// Simple ARM7 stub (sends RTC, TSC, and X/Y data to the ARM 9)
// -- joat
//////////////////////////////////////////////////////////////////////

#include <NDS/NDS.h>

#include <NDS/ARM7/BIOS.h>
#include <NDS/ARM7/touch.h>
#include <NDS/ARM7/clock.h>

//////////////////////////////////////////////////////////////////////

void InterruptHandler(void) {
  static int heartbeat = 0;

  if (IF & IRQ_VBLANK) {
    uint16 but = 0, x = 0, y = 0, z1 = 0, z2 = 0;
    int t1 = 0, t2 = 0;
    uint32 temp = 0;
    uint32 i = 0;
    uint8 ct[sizeof(IPC->curtime)];
    // Update the heartbeat
    heartbeat++;

    // Read the X/Y buttons and the /PENIRQ line
    but = XKEYS;
    if (!(but & 0x40)) {
      // Read the touch screen
      x = touchRead(TSC_MEASURE_X);
      y = touchRead(TSC_MEASURE_Y);
      z1 = touchRead(TSC_MEASURE_Z1);
      z2 = touchRead(TSC_MEASURE_Z2);
    }
    // Read the time
    rtcGetTime((uint8 *)ct);
    BCDToInteger((uint8 *)&(ct[1]), 7);

    // Read the temperature
    temp = touchReadTemperature(&t1, &t2);

    // Update the IPC struct
    IPC->heartbeat = heartbeat;
    IPC->buttons = but;
    IPC->touchX = x;
    IPC->touchY = y;
    IPC->touchZ1 = z1;
    IPC->touchZ2 = z2;
    for(i = 0; i < sizeof(ct); ++i) {
      IPC->curtime[i]=ct[i];
    }
    IPC->temperature = temp;
    IPC->tdiode1 = t1;
    IPC->tdiode2 = t2;
  }

  // Acknowledge interrupts
  IF = IF;
}

//////////////////////////////////////////////////////////////////////

int main(int argc, char ** argv) {
  // Reset the clock if needed
  rtcReset();

  // Set up the interrupt handler
  IME = 0;
  IRQ_HANDLER = &InterruptHandler;
  IE = IRQ_VBLANK;
  IF = ~0;
  DISP_SR = DISP_VBLANK_IRQ;
  IME = 1;

  // Keep the ARM7 out of main RAM
  while (1) swiWaitForVBlank();
  return 0;
}

//////////////////////////////////////////////////////////////////////
